@@ -1,5 +1,6 @@
 ;;; gnus-logic.el --- advanced scoring code for Gnus
-;; Copyright (C) 1996,97,98 Free Software Foundation, Inc.
+;; Copyright (C) 1996, 1997, 1998, 1999, 2000
+;;        Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; Keywords: news
@@ -171,9 +172,9 @@
      ((eq type 'at)
       (equal date match))
      ((eq type 'before)
-      (gnus-time-less match date))
+      (time-less-p match date))
      ((eq type 'after)
-      (gnus-time-less date match))
+      (time-less-p date match))
      (t
       (error "No such date score type: %s" type)))))
 
@@ -218,7 +219,7 @@
 		      ((memq type '(s S string String))
 		       'search-forward)
 		      (t
-		       (error "Illegal match type: %s" type)))))
+		       (error "Invalid match type: %s" type)))))
 	  (goto-char (point-min))
 	  (prog1
 	      (funcall search-func match nil t)
